library(ggplot2)
library(data.table)

gs <- read.delim("student_information.tsv")
d <- gs[,c(2,5)]
colnames(d) <- c("student.num", "discord.name")

call.list <- do.call("rbind", lapply(list.files(".", pattern="^call_list-.*tsv$"), function (x) {read.delim(x)[,1:4]}))
colnames(call.list) <- gsub("_", ".", colnames(call.list))

call.list$day <- as.Date(call.list$timestamp)

## drop calls where the person wasn't present
call.list.full <- call.list
call.list[!call.list$answered,]
call.list <- call.list[call.list$answered,]

call.counts <- data.frame(table(call.list$discord.name))
colnames(call.counts) <- c("discord.name", "num.calls")

d <- merge(d, call.counts, all.x=TRUE, all.y=TRUE, by="discord.name"); d

## set anything that's missing to zero
d$num.calls[is.na(d$num.calls)] <- 0
      
attendance <- unlist(lapply(list.files(".", pattern="^attendance-.*tsv$"), function (x) {d <- read.delim(x); strsplit(d[[2]], ",")}))

file.to.attendance.list <- function (x) {
    tmp <- read.delim(x)
    d.out <- data.frame(discord.name=unlist(strsplit(tmp[[2]], ",")))
    d.out$day <- rep(as.Date(tmp[[1]][1]), nrow(d.out))
    return(d.out)
}

attendance <- do.call("rbind",
                      lapply(list.files(".", pattern="^attendance-.*tsv$"),
                             file.to.attendance.list))

## create list of folks who are missing in class 
missing.in.class  <- call.list.full[is.na(call.list.full$answered) |
                                    (!is.na(call.list.full$answered) & !call.list.full$answered),
                                    c("discord.name", "day")]

missing.in.class <- unique(missing.in.class)

setDT(attendance)
setkey(attendance, discord.name, day)
setDT(missing.in.class)
setkey(missing.in.class, discord.name, day)

## drop presence for people on missing days
attendance[missing.in.class,]
attendance <- as.data.frame(attendance[!missing.in.class,])

attendance.counts <- data.frame(table(attendance$discord.name))
colnames(attendance.counts) <- c("discord.name", "num.present")

d <- merge(d, attendance.counts,
           all.x=TRUE, all.y=TRUE,
           by="discord.name")

days.list <- lapply(unique(attendance$day), function (day) {
    day.total <- table(call.list.full$day == day)[["TRUE"]]
    lapply(d$discord.name, function (discord.name) {
        num.present <- nrow(attendance[attendance$day == day & attendance$discord.name == discord.name,])
        if (num.present/day.total > 1) {print(day)}
        data.frame(discord.name=discord.name,
                   days.present=(num.present/day.total))
    })
})

days.tmp <- do.call("rbind", lapply(days.list, function (x) do.call("rbind", x)))

days.tbl <- tapply(days.tmp$days.present, days.tmp$discord.name, sum)

attendance.days <- data.frame(discord.name=names(days.tbl),
                              days.present=days.tbl,
                              days.absent=length(list.files(".", pattern="^attendance-.*tsv$"))-days.tbl)

d <- merge(d, attendance.days,
           all.x=TRUE, all.y=TRUE, by="discord.name")

d[sort.list(d$days.absent), c("discord.name", "num.calls", "days.absent")]

## make some visualizations of whose here/not here
#######################################################

png("questions_absence_histogram_combined.png", units="px", width=800, height=600)

ggplot(d) +
    aes(x=as.factor(num.calls), fill=days.absent, group=days.absent) +
    geom_bar(color="black") +
    scale_x_discrete("Number of questions asked") +
    scale_y_continuous("Number of students") +
    scale_fill_continuous("Days absent", low="red", high="blue")+
    theme_bw()

dev.off()

png("questions_absenses_boxplots.png", units="px", width=800, height=600)

ggplot(data=d) +
    aes(x=as.factor(num.calls), y=days.absent) +
    geom_boxplot() +
    scale_x_discrete("Number of questions asked") +
    scale_y_continuous("Days absent")

dev.off()

